
package uoc.ows.bbdd;

import uoc.ows.common.*;
import java.io.InputStream;
import java.io.IOException;
import java.sql.*;
import java.util.Properties;
import java.sql.Statement;


/** 
 * Gestio generica de connexions, desconnexions i interacci amb BBDD
 * @author Lluis Cozar Prez
 * @version 1.0
 */

public class GestorDiscConnexio {

    protected Connection conexion = null;
   
   /** 
    * ConnectarBD Metode de creaci connexio a la BBDD. Credencials parametritzades a configuration.properties.txt 
    * @throws OWSException
    */
    public void conectarBD() throws OWSException {
        String url = "";
        String usuario = "";
        String clave ="";
         
        try {
            Class.forName("com.mysql.jdbc.Driver");         
        } catch (ClassNotFoundException e) {            
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DRV);
        }
          
        try {
            Properties prop = new Properties();
            
            ClassLoader cl = this.getClass().getClassLoader();
            InputStream is = cl.getResourceAsStream("properties/configuration.properties");
            
            prop.load(is);
            
            url = prop.getProperty("url");
            usuario = prop.getProperty("username");
            clave = prop.getProperty("password");
            
            
        } catch (java.io.FileNotFoundException e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_NOT_FILE);
        } catch (IOException ioe) {
            System.out.println(ioe.getMessage());
            throw new OWSException(OWSException.ERR_PROPERTIES);
        }
       
        try {
            conexion = DriverManager.getConnection(url, usuario, clave);
        } catch (SQLException sqle) {
            System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_CONECTION);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_CONECTION);
        }
        
    }

    /**
     * crearSentencia() Creaci Statement per manipulaci de BBDD
     * {pre: existencia connexio a BD
     * {post: creaci de Statement 
     * @return Statement
     * @throws OWSException
     */
    public Statement crearSentencia() throws OWSException {
        Statement sentencia = null;
        try {
            sentencia = conexion.createStatement();
        } catch (SQLException sqle) {
            System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_CONECTION);
        }
        return sentencia;
    }

    /**
     * crearPreparedStatement Creacio PreparedStatement per manipulaci de BBDD
     * {pre: existencia connexio a BD
     * {post: creacio de PreparedStatement
     * @return PreparedStatement
     * @throws OWSException
     */
    public PreparedStatement crearPreparedStatement(String ps) throws OWSException {
        try {
            return conexion.prepareStatement(ps, ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
        } catch (SQLException sqle) {
            System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_CONECTION);
        }
    }
   
    /** 
     * cerrarConnexionBD Metode de tancament connexio a la BBDD.  
     * @throws OWSException
     */
    public void cerrarConexionBD() throws OWSException {
        try {
            conexion.close(); 
        } catch (SQLException sqle) {
            System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_BD_STOP);
        } catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_BD_STOP);
        }
    }

    /**
     * Recuperacio de la connexio actual
     * @return connexio actual
     */
    public Connection getConexion() {
        return conexion;
    }
    
    /**
     * Setter de la connexio
     * @param pConexion
     */
    public void setConexion(Connection pConexion) {
        conexion = pConexion;
    }
    
}
