package uoc.ows.bbdd;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.sql.Timestamp;


import uoc.ows.beans.Execution;
import uoc.ows.common.OWSException;


/**
 * Gestio d'acces a la taula d'execuions.
 *
 * @author Lluis Cozar Prez
 */
public class GestorExecutions extends GestorDiscConnexio {

	/**
	 * Instantiates a new gestor executions.
	 *
	 * @throws OWSException the oWS exception
	 */
	public GestorExecutions() throws OWSException {
		super.conectarBD();
	}
	
	/**
	 * Instantiates a new gestor executions.
	 *
	 * @param con the con
	 * @throws OWSException the oWS exception
	 */
	public GestorExecutions(Connection con) throws OWSException {
		conexion = con;
	}
	
	/**
	 * Insert line execution.
	 *
	 * @param executionLine the execution line
	 * @throws OWSException the oWS exception
	 */
	public void insertLineExecution(Execution executionLine) throws OWSException {

		if (conexion == null)
			conexion = getConexion();

		try {
			conexion.setAutoCommit(false);
			
			String sql = "INSERT executions (idjob, offsetjob, position, line, timestamp) VALUES (?, ?, ?, ?, ?) ";
			PreparedStatement pst = conexion.prepareStatement(sql);

			pst.clearParameters();
			pst.setInt(1, executionLine.getIdjob());
			pst.setString(2, executionLine.getOffsetjob());
			pst.setInt(3, executionLine.getPosition());
			pst.setString(4, executionLine.getLine());
			pst.setTimestamp(5, new java.sql.Timestamp(executionLine.getTimestamp().getTime()));
			
			try {
				pst.executeUpdate();
			} catch (SQLException sqle) {
				System.out.println(sqle.getMessage());
				conexion.rollback();
				throw new OWSException(OWSException.ERR_SQL);
			}
			
			conexion.commit();
			conexion.setAutoCommit(true);
			pst.close();
			
		} catch (SQLException sqle) {
			System.out.println(sqle.getMessage());
			throw new OWSException(OWSException.ERR_SQL);
		} catch (Exception e) {
			System.out.println(e.getMessage());
			throw new OWSException(OWSException.ERR_DESC_EXEC);
		}
		
	}

	/**
	 * Gets the executions.
	 *
	 * @param idjob the idjob
	 * @return the executions
	 * @throws OWSException the oWS exception
	 */
	public List<Execution> getExecutions(int idjob) throws OWSException{
		List<Execution> executions = new ArrayList<Execution>();
		
		ResultSet rs;
		PreparedStatement pst;
		
		if (conexion==null) conexion = getConexion();
		
		String sql = "SELECT timestamp, idjob FROM executions" +
                     " WHERE idjob = ? AND POSITION = 1 GROUP BY timestamp, idjob";
		
		try{
			pst = crearPreparedStatement(sql);
			pst.clearParameters();
			pst.setInt(1, idjob);
			rs = pst.executeQuery();
			 
			while(rs.next()){
				Execution execution = new Execution();
				execution.setIdjob(rs.getInt("idjob"));
				execution.setTimestamp(rs.getTimestamp("timestamp"));
				executions.add(execution);
			}
			pst.close();
			rs.close();
			
		}catch(SQLException sqle){
			System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
		}catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DESC_EXEC);
        }
				
		return executions;
	}

	/**
	 * Gets the execution.
	 *
	 * @param idjob the idjob
	 * @param timestamp the timestamp
	 * @return the execution
	 * @throws OWSException the oWS exception
	 */
	public List<Execution> getExecution(int idjob, String timestamp) throws OWSException{
		List<Execution> executions = new ArrayList<Execution>();
		
		ResultSet rs;
		PreparedStatement pst;
		
		if (conexion==null) conexion = getConexion();
		
		String sql = "SELECT timestamp, idjob, line FROM executions" +
                     " WHERE idjob = ? AND timestamp = ? ORDER BY position";
		
		try{
			pst = crearPreparedStatement(sql);
			pst.clearParameters();
			pst.setInt(1, idjob);
			
			
			pst.setTimestamp(2, Timestamp.valueOf(timestamp) );
			rs = pst.executeQuery();
			 
			while(rs.next()){
				Execution execution = new Execution();
				execution.setLine(rs.getString("line"));
				execution.setTimestamp(rs.getTimestamp("timestamp"));
				System.out.println(execution.getLine());
				executions.add(execution);
			}
			pst.close();
			rs.close();
			
		}catch(SQLException sqle){
			System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
		}catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DESC_EXEC);
        }
				
		return executions;
	}
}
