package uoc.ows.bbdd;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import uoc.ows.beans.Job;
import uoc.ows.common.OWSException;
import uoc.ows.common.Constants;
import uoc.ows.bbdd.GestorNodes;

/**
 * Gestio d'acces a la taula de jobs.
 *
 * @author Lluis Cozar Prez
 */

public class GestorJobs extends GestorDiscConnexio {

	/**
	 * Instantiates a new gestor jobs.
	 *
	 * @throws OWSException the oWS exception
	 */
	public GestorJobs() throws OWSException {
		super.conectarBD();
	}
	
	/**
	 * Instantiates a new gestor jobs.
	 *
	 * @param con the con
	 * @throws OWSException the oWS exception
	 */
	public GestorJobs(Connection con) throws OWSException {
		conexion = con;
	}
	
	/**
	 * Schedule job.
	 *
	 * @param job the job
	 * @throws OWSException the oWS exception
	 */
	public void scheduleJob(Job job) throws OWSException {

		if (conexion == null)
			conexion = getConexion();

		try {
			conexion.setAutoCommit(false);
			
			String sql = "INSERT jobs (idnode, idestado, executionCmd, executionEnv, executionParm, insertTimestamp, pattern) VALUES (?, ?, ?, ?, ?, now(), ?) ";
			PreparedStatement pst = conexion.prepareStatement(sql);

			pst.clearParameters();
			pst.setInt(1, job.getIdnode());
			pst.setString(2, job.getIdstatus());
			pst.setString(3, job.getExecutionCmd());
			pst.setString(4,  job.getExecutionEnv());
			pst.setString(5,  job.getExecutionParm());
			pst.setString(6, job.getPattern());
			
			try {
				pst.executeUpdate();
			} catch (SQLException sqle) {
				System.out.println(sqle.getMessage());
				conexion.rollback();
				throw new OWSException(OWSException.ERR_SQL);
			}
			
			conexion.commit();
			conexion.setAutoCommit(true);
			pst.close();
			
		} catch (SQLException sqle) {
			System.out.println(sqle.getMessage());
			throw new OWSException(OWSException.ERR_SQL);
		} catch (Exception e) {
			System.out.println(e.getMessage());
			throw new OWSException(OWSException.ERR_DESC_EXEC);
		}
		
	}

	/**
	 * Gets the pending jobs.
	 *
	 * @return the pending jobs
	 * @throws OWSException the oWS exception
	 */
	public List<Job> getPendingJobs() throws OWSException{
		List<Job> jobsPending = new ArrayList<Job>();
		
		ResultSet rs;
		PreparedStatement pst;
		
		if (conexion==null) conexion = getConexion();
		
		String sql = "SELECT idjob, idnode, idestado, offsetjob, executionCMD, executionEnv, executionParm, lastRC, " +
				" numberofTransitions, numberofExecutions, insertTimestamp, lastUpdateTimeStamp, pattern" +
                     " FROM jobs WHERE idestado = '" + Constants.PENDING + "'" + 
				     " OR idestado = '" + Constants.DESCHEDULING + "'" + 
				     " OR idestado = '" + Constants.RESCHEDULING + "'"; 
		
		try{
			pst = crearPreparedStatement(sql);
			pst.clearParameters();
			rs = pst.executeQuery();
			 
			while(rs.next()){
				Job job = new Job();
				job.setIdjob(rs.getInt("idjob"));
				job.setIdnode(rs.getInt("idnode"));
				job.setIdstatus(rs.getString("idestado"));
				job.setOffsetjob(rs.getString("offsetjob"));
				job.setExecutionCmd(rs.getString("executionCmd"));
				job.setExecutionEnv(rs.getString("executionEnv"));
				job.setExecutionParm(rs.getString("executionParm"));
				job.setLastRC(rs.getInt("lastRC"));
				job.setNumberofTransitions(rs.getInt("numberofTransitions"));
				job.setNumberofExecutions(rs.getInt("numberofExecutions"));
				job.setLastUpdateTimestamp(rs.getDate("lastUpdateTimestamp"));
				job.setInsertTimestamp(rs.getDate("insertTimestamp"));
				job.setPattern(rs.getString("pattern"));
				jobsPending.add(job);
			}
			pst.close();
			rs.close();
			
		}catch(SQLException sqle){
			System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
		}catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DESC_EXEC);
        }
				
		return jobsPending;
	}
	
	/**
	 * Gets the job.
	 *
	 * @param idjob the idjob
	 * @return the job
	 * @throws OWSException the oWS exception
	 */
	public Job getJob(int idjob) throws OWSException{
		Job job = new Job();
		ResultSet rs;
		PreparedStatement pst;
		
		if (conexion==null) conexion = getConexion();
		
		String sql = "SELECT idjob, idnode, idestado, offsetjob, executionCMD, executionEnv, executionParm, lastRC, " +
				" numberofTransitions, numberofExecutions, insertTimestamp, lastUpdateTimeStamp, pattern" +
                     " FROM jobs WHERE idjob = ?";
		
		try{
			pst = crearPreparedStatement(sql);
			pst.clearParameters();
			pst.setInt(1, idjob);
			rs = pst.executeQuery();
			while(rs.next()) 
			{
				job.setIdjob(rs.getInt("idjob"));
				job.setIdnode(rs.getInt("idnode"));
				job.setIdstatus(rs.getString("idestado"));
				job.setOffsetjob(rs.getString("offsetjob"));
				job.setExecutionCmd(rs.getString("executionCmd"));
				job.setExecutionEnv(rs.getString("executionEnv"));
				job.setExecutionParm(rs.getString("executionParm"));
				job.setLastRC(rs.getInt("lastRC"));
				job.setNumberofTransitions(rs.getInt("numberofTransitions"));
				job.setNumberofExecutions(rs.getInt("numberofExecutions"));
				job.setLastUpdateTimestamp(rs.getTimestamp("lastUpdateTimestamp"));
				job.setInsertTimestamp(rs.getTimestamp("insertTimestamp"));
				job.setPattern(rs.getString("pattern"));
				
			}
			pst.close();
			rs.close();
			
		}catch(SQLException sqle){
			System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
		}catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DESC_EXEC);
        }
				
		return job;
	}
	
	/**
	 * Update job.
	 *
	 * @param job the job
	 * @throws OWSException the oWS exception
	 */
	public void updateJob(Job job) throws OWSException {

		if (conexion == null)
			conexion = getConexion();

		try {
			conexion.setAutoCommit(false);
			String sql = "UPDATE JOBS SET "
					+ "idestado = ?, offsetjob = ?, lastRC = ?, "
					+ "numberofTransitions = ?, numberofExecutions = ?, pattern = ?, lastUpdateTimestamp = now()"
					+ " WHERE idjob = ?";
			PreparedStatement pst = conexion.prepareStatement(sql);

			pst.clearParameters();
			pst.setString(1, job.getIdstatus());
			pst.setString(2, job.getOffsetjob());
			pst.setInt(3, job.getLastRC());
			pst.setInt(4, (job.getNumberofTransitions()+1));
			pst.setInt(5, job.getNumberofExecutions());
			pst.setString(6, job.getPattern());
			pst.setInt(7, job.getIdjob());

			try {
				pst.executeUpdate();
			} catch (SQLException sqle) {
				System.out.println(sqle.getMessage());
				conexion.rollback();
				throw new OWSException(OWSException.ERR_SQL);
			}
			
			conexion.commit();
			conexion.setAutoCommit(true);
			pst.close();
			
		} catch (SQLException sqle) {
			System.out.println(sqle.getMessage());
			throw new OWSException(OWSException.ERR_SQL);
		} catch (Exception e) {
			System.out.println(e.getMessage());
			throw new OWSException(OWSException.ERR_DESC_EXEC);
		}
		
	}
	
	/**
	 * * web **.
	 *
	 * @return the jobs
	 * @throws OWSException the oWS exception
	 */ 
	
	public List<Job> getJobs() throws OWSException{
		List<Job> jobs = new ArrayList<Job>();
		
		ResultSet rs;
		PreparedStatement pst;
		
		if (conexion==null) conexion = getConexion();
		
		String sql = "SELECT idjob, idnode, idestado, offsetjob, executionCMD, executionEnv, executionParm, lastRC, " +
				" numberofTransitions, numberofExecutions, insertTimestamp, lastUpdateTimeStamp, pattern" +
                     " FROM jobs";
		
		try{
			pst = crearPreparedStatement(sql);
			pst.clearParameters();
			rs = pst.executeQuery();
			 
			while(rs.next()){
				Job job = new Job();
				job.setIdjob(rs.getInt("idjob"));
				job.setIdnode(rs.getInt("idnode"));
				job.setNode(new GestorNodes(this.conexion).getNodeDNS(job.getIdnode()));
				job.setIdstatus(rs.getString("idestado"));
				job.setOffsetjob(rs.getString("offsetjob"));
				job.setExecutionCmd(rs.getString("executionCmd"));
				job.setExecutionEnv(rs.getString("executionEnv"));
				job.setExecutionParm(rs.getString("executionParm"));
				job.setLastRC(rs.getInt("lastRC"));
				job.setNumberofTransitions(rs.getInt("numberofTransitions"));
				job.setNumberofExecutions(rs.getInt("numberofExecutions"));
				job.setLastUpdateTimestamp(rs.getDate("lastUpdateTimestamp"));
				job.setInsertTimestamp(rs.getTimestamp("insertTimestamp"));
				job.setPattern(rs.getString("pattern"));
				jobs.add(job);
			}
			pst.close();
			rs.close();
			
		}catch(SQLException sqle){
			System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
		}catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DESC_EXEC);
        }
				
		return jobs;
	}

}
