package uoc.ows.bbdd;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import uoc.ows.common.OWSException;

/**
 * Gestio d'acces a la taula de nodes.
 *
 * @author Lluis Cozar Prez
 */
public class GestorNodes extends GestorDiscConnexio {

	/**
	 * Instantiates a new gestor nodes.
	 *
	 * @throws OWSException the oWS exception
	 */
	public GestorNodes() throws OWSException {
		super.conectarBD();
	}
	
	/**
	 * Instantiates a new gestor nodes.
	 *
	 * @param con the con
	 * @throws OWSException the oWS exception
	 */
	public GestorNodes(Connection con) throws OWSException {
		conexion = con;
	}
	
	/**
	 * Gets the node dns.
	 *
	 * @param idnode the idnode
	 * @return the node dns
	 * @throws OWSException the oWS exception
	 */
	public String getNodeDNS(int idnode) throws OWSException{
		String nodeName = "";
		ResultSet rs;
		PreparedStatement pst;
		
		if (conexion==null) conexion = getConexion();
		
		String sql = "SELECT ipadress" +
                     " FROM nodes WHERE idnode = ?" ;
		
		try{
			pst = crearPreparedStatement(sql);
			pst.clearParameters();
			pst.setInt(1, idnode);
			rs = pst.executeQuery();
			 
			while(rs.next()){
				nodeName = rs.getString("ipadress");
			}
			pst.close();
			rs.close();
			
		}catch(SQLException sqle){
			System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
		}catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DESC_EXEC);
        }
				
		return nodeName;
	}

	/**
	 * Gets the id node.
	 *
	 * @param nodeName the node name
	 * @return the id node
	 * @throws OWSException the oWS exception
	 */
	public int getidNode(String nodeName) throws OWSException{
		int idnode = 0;
		ResultSet rs;
		PreparedStatement pst;
		
		if (conexion==null) conexion = getConexion();
		
		String sql = "SELECT idnode" +
                     " FROM nodes WHERE ipadress = ?" ;
		
		try{
			pst = crearPreparedStatement(sql);
			pst.clearParameters();
			pst.setString(1, nodeName);
			rs = pst.executeQuery();
			 
			while(rs.next()){
				idnode = rs.getInt("idnode");
			}
			pst.close();
			rs.close();
			
		}catch(SQLException sqle){
			System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
		}catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DESC_EXEC);
        }
				
		return idnode;
	}

}
