package uoc.ows.bbdd;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import uoc.ows.beans.Job;
import uoc.ows.beans.Transition;
import uoc.ows.common.OWSException;

/**
 * Gestio d'acces a la taula de transicions.
 *
 * @author Lluis Cozar Prez
 */
public class GestorTransitions extends GestorDiscConnexio {

	/**
	 * Instantiates a new gestor transitions.
	 *
	 * @throws OWSException the oWS exception
	 */
	public GestorTransitions() throws OWSException {
		super.conectarBD();
	}
	
	/**
	 * Instantiates a new gestor transitions.
	 *
	 * @param con the con
	 * @throws OWSException the oWS exception
	 */
	public GestorTransitions(Connection con) throws OWSException {
		conexion = con;
	}
	
	/**
	 * Insert transition.
	 *
	 * @param job the job
	 * @throws OWSException the oWS exception
	 */
	public void insertTransition(Job job) throws OWSException {

		if (conexion == null)
			conexion = getConexion();

		try {
			conexion.setAutoCommit(false);
			
			String sql = "INSERT transacciones (idjob, idestado, timestamp, offsetjob) VALUES (?, ?, now(), ?) ";
			PreparedStatement pst = conexion.prepareStatement(sql);

			pst.clearParameters();
			pst.setInt(1, job.getIdjob());
			pst.setString(2, job.getIdstatus());
			pst.setString(3,  job.getOffsetjob());
		
			try {
				pst.executeUpdate();
			} catch (SQLException sqle) {
				System.out.println(sqle.getMessage());
				conexion.rollback();
				throw new OWSException(OWSException.ERR_SQL);
			}
			
			conexion.commit();
			conexion.setAutoCommit(true);
			pst.close();
			
		} catch (SQLException sqle) {
			System.out.println(sqle.getMessage());
			throw new OWSException(OWSException.ERR_SQL);
		} catch (Exception e) {
			System.out.println(e.getMessage());
			throw new OWSException(OWSException.ERR_DESC_EXEC);
		}
		
	}
	
	/**
	 * Gets the transitions.
	 *
	 * @param idjob the idjob
	 * @return the transitions
	 * @throws OWSException the oWS exception
	 */
	public List<Transition> getTransitions(int idjob) throws OWSException{
		List<Transition> transitions = new ArrayList<Transition>();
		
		ResultSet rs;
		PreparedStatement pst;
		
		if (conexion==null) conexion = getConexion();
		
		String sql = "SELECT idjob, idestado, timestamp, offsetjob " +
                     " FROM transacciones WHERE idjob = ?";
		
		try{
			pst = crearPreparedStatement(sql);
			pst.clearParameters();
			pst.setInt(1, idjob);
			rs = pst.executeQuery();
			 
			while(rs.next()){
				Transition transition = new Transition();
				transition.setIdjob(rs.getInt("idjob"));
				transition.setIdstatus(rs.getString("idestado"));
				transition.setTimestamp(rs.getTimestamp("timestamp"));
				transition.setOffsetjob(rs.getString("offsetjob"));
				transitions.add(transition);
			}
			pst.close();
			rs.close();
			
		}catch(SQLException sqle){
			System.out.println(sqle.getMessage());
            throw new OWSException(OWSException.ERR_SQL);
		}catch (Exception e) {
            System.out.println(e.getMessage());
            throw new OWSException(OWSException.ERR_DESC_EXEC);
        }
				
		return transitions;
	}

}
