package uoc.ows.bbdd;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import uoc.ows.beans.Execution;
import uoc.ows.common.OWSException;

/**
 * Gestio d'acces a la taula d'execuions.
 *
 * @author Lluis Cozar Prez
 */

public class GestorExecutions extends GestorDiscConnexio {

	/**
	 * Instantiates a new gestor executions.
	 *
	 * @throws OWSException the oWS exception
	 */
	public GestorExecutions() throws OWSException {
		
	}
	
	/**
	 * Instantiates a new gestor executions.
	 *
	 * @param con the con
	 * @throws OWSException the oWS exception
	 */
	public GestorExecutions(Connection con) throws OWSException {
		conexion = con;
	}
	
	/**
	 * Insert line execution.
	 *
	 * @param executionLine the execution line
	 * @throws OWSException the oWS exception
	 */
	public void insertLineExecution(Execution executionLine) throws OWSException {

		if (conexion == null)
			conexion = getConexion();

		try {
			conexion.setAutoCommit(false);
			
			String sql = "INSERT executions (idjob, offsetjob, position, line, timestamp) VALUES (?, ?, ?, ?, ?) ";
			PreparedStatement pst = conexion.prepareStatement(sql);

			pst.clearParameters();
			pst.setInt(1, executionLine.getIdjob());
			pst.setString(2, executionLine.getOffsetjob());
			pst.setInt(3, executionLine.getPosition());
			pst.setString(4, executionLine.getLine());
			pst.setTimestamp(5, new java.sql.Timestamp(executionLine.getTimestamp().getTime()));
			
			try {
				pst.executeUpdate();
			} catch (SQLException sqle) {
				System.out.println(sqle.getMessage());
				conexion.rollback();
				throw new OWSException(OWSException.ERR_SQL);
			}
			
			conexion.commit();
			conexion.setAutoCommit(true);
			pst.close();
			
		} catch (SQLException sqle) {
			System.out.println(sqle.getMessage());
			throw new OWSException(OWSException.ERR_SQL);
		} catch (Exception e) {
			System.out.println(e.getMessage());
			throw new OWSException(OWSException.ERR_DESC_EXEC);
		}
		
	}

}
