package uoc.ows.client;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import javax.jms.JMSException;

import uoc.ows.bbdd.GestorJobs;
import uoc.ows.bbdd.GestorNodes;
import uoc.ows.bbdd.GestorTransitions;
import uoc.ows.common.Constants;
import uoc.ows.common.OWSException;
import uoc.ows.beans.Job;
 
/**
 * CommandLineAPIScheduler
 * Utilitat de linea de comandament per la planificaci, desplanificaci i modificaci del calendari d'un treball.
 *
 * @author LLuis Cozar Prez
 */

public class CommandLineAPIScheduler {
 
	/** The gn. */
	private static GestorNodes gn;
	
	/** The gj. */
	private static GestorJobs gj;
	
	/** The gt. */
	private static GestorTransitions gt;
	
	/**
	 * The main method.
	 *
	 * @param args 
	 * @throws Exception 
	 */
	public static void main(String[] args) throws Exception {

		String file = "";
		if (args.length > 0) {

			file = args[1];
			if ((args[0].compareTo("help")) == 0) {
				System.out.println("use java class sched/desch/modify file");
			} else if ((args[0].compareTo("sch")) == 0) {
				scheduling(file);				
			} else if (args[0].compareTo("desch") == 0) {
				descheduling(file);	
			} else if (args[0].compareTo("modify") == 0) {
				modify(file);
			} else {				
				System.out.println("use java class sch/desch file");
			}
		}
	}
	
	/**
	 * Scheduling.
	 *
	 * @param file 
	 * @throws JMSException 
	 * @throws OWSException 
	 */
	private static void scheduling(String file) throws JMSException, OWSException {
		
		try {
			Properties prop = new Properties();
			final String propertiesFile = new File("")
					.getAbsolutePath() + file;
			System.out.println(propertiesFile);
			prop.load(new FileInputStream(propertiesFile));

			gn = new GestorNodes();
			gn.conectarBD();

			Job wrkJob = new Job();
			wrkJob.setIdnode(gn.getidNode(prop.getProperty("node")));
			wrkJob.setExecutionCmd(prop.getProperty("executionCmd"));
			wrkJob.setExecutionEnv(prop.getProperty("executionEnv"));
			wrkJob.setExecutionParm(prop.getProperty("executionParms"));
			wrkJob.setPattern(prop.getProperty("pattern"));
			wrkJob.setIdstatus(Constants.PENDING);
			wrkJob.setOffsetjob("0");

			if (wrkJob.getIdnode() != 0) {
				gj = new GestorJobs(gn.getConexion());
				gj.scheduleJob(wrkJob);
				System.out.println("Job incorporat correctament! >> "
						+ wrkJob.toString());
			} else {
				System.out
						.println("Reviseu el fitxer de planificaci. Node inexistent!");
			}
			gn.cerrarConexionBD();

		} catch (java.io.FileNotFoundException e) {
			System.out.println(e.getMessage());
			throw new OWSException(OWSException.ERR_NOT_FILE);
		} catch (IOException ioe) {
			System.out.println(ioe.getMessage());
			throw new OWSException(OWSException.ERR_PROPERTIES);
		}

	}
	
	/**
	 * Descheduling.
	 *
	 * @param file 
	 * @throws JMSException 
	 * @throws OWSException 
	 */
	private static void descheduling(String file) throws JMSException, OWSException {
		try {
			Properties prop = new Properties();
			final String propertiesFile = new File("")
					.getAbsolutePath() + file;
			System.out.println(propertiesFile);
			prop.load(new FileInputStream(propertiesFile));

			gj = new GestorJobs();
			gj.conectarBD();
			
			Job wrkJob = gj.getJob(Integer.parseInt(prop
					.getProperty("idjob")));
			wrkJob.setIdstatus(Constants.DESCHEDULING);

			if (wrkJob.getIdjob() != 0) {
				System.out.println("Solicitud de desincorporaci job correcta! >> "
						+ wrkJob.toString());
				gt = new GestorTransitions(gj.getConexion());
				gj.updateJob(wrkJob);
				gt.insertTransition(wrkJob);
			} else {
				System.out
						.println("Reviseu el fitxer de planificaci. Idjob inexistent!");
			}

			gj.cerrarConexionBD();

		} catch (java.io.FileNotFoundException e) {
			System.out.println(e.getMessage());
			throw new OWSException(OWSException.ERR_NOT_FILE);
		} catch (IOException ioe) {
			System.out.println(ioe.getMessage());
			throw new OWSException(OWSException.ERR_PROPERTIES);
		}
	}
	
	/**
	 * Modify.
	 *
	 * @param file 
	 * @throws JMSException 
	 * @throws OWSException 
	 */
	private static void modify(String file) throws JMSException, OWSException {
		
		try {
			Properties prop = new Properties();
			final String propertiesFile = new File("")
					.getAbsolutePath() + file;
			System.out.println(propertiesFile);
			prop.load(new FileInputStream(propertiesFile));

			gj = new GestorJobs();
			gj.conectarBD();
			
			Job wrkJob = gj.getJob(Integer.parseInt(prop
					.getProperty("idjob")));
			wrkJob.setIdstatus(Constants.RESCHEDULING);
			wrkJob.setPattern(prop
					.getProperty("pattern"));

			if (wrkJob.getIdjob() != 0) {
				System.out.println("Solicitud de replanificaci job correcta! >> "
						+ wrkJob.toString());
				gt = new GestorTransitions(gj.getConexion());
				gj.updateJob(wrkJob);
				gt.insertTransition(wrkJob);
			} else {
				System.out
						.println("Reviseu el fitxer de planificaci. Idjob inexistent!");
			}

			gj.cerrarConexionBD();

		} catch (java.io.FileNotFoundException e) {
			System.out.println(e.getMessage());
			throw new OWSException(OWSException.ERR_NOT_FILE);
		} catch (IOException ioe) {
			System.out.println(ioe.getMessage());
			throw new OWSException(OWSException.ERR_PROPERTIES);
		}
	}
	
}
    