package uoc.ows.nodeclient;

import org.apache.activemq.ActiveMQConnection;

import org.apache.activemq.ActiveMQConnectionFactory;
 
import javax.jms.*;

import java.util.HashMap;
import java.util.Map;
import java.util.logging.ConsoleHandler;
import java.util.logging.Logger;

import uoc.ows.beans.MessageSimpleMQ;
import uoc.ows.nodeclient.StartExec;
import uoc.ows.server.ServerManager;
import uoc.ows.common.Constants;
import uoc.ows.common.CustomRecordFormatter;

/**
 * ClientManager
 * s tracta de la classe principal que dona forma al node (receptor de missatges i executor del treball en el agent)
 * Treballa a mode de daemon i en cada cicle intercepta els missatges rebuts del scheduler, executa la operaci i retorna
 * la execuci (stdout/err) i registre de finalitzaci.
 *
 * @author LLuis Cozar Prez
 */

public class ClientManager {

	/** The logger. */
	private static Logger logger = Logger.getLogger(ServerManager.class.getName());
	
    /** The Constant URL. */
    private static final String URL = "tcp://localhost:61616";
 
    /** The Constant USER. */
    private static final String USER = ActiveMQConnection.DEFAULT_USER;
 
    /** The Constant PASSWORD. */
    private static final String PASSWORD = ActiveMQConnection.DEFAULT_PASSWORD;
 
    /** The Constant DESTINATION_QUEUE. */
    private static final String DESTINATION_QUEUE = Constants.qname;
 
    /** The Constant TRANSACTED_SESSION. */
    private static final boolean TRANSACTED_SESSION = false;
 
    /** The Constant TIMEOUT. */
    private static final int TIMEOUT = 1000;
 
    /** The consumed message types. */
    private final Map<String, Integer> consumedMessageTypes;
 
    /** The total consumed messages. */
    private int totalConsumedMessages = 0;
 
    /**
     * Instantiates a new client manager.
     */
    public ClientManager() {
        this.consumedMessageTypes = new HashMap<String, Integer>();
    }
 
    /**
     * Process messages.
     *
     * @throws JMSException the jMS exception
     */
    public void processMessages() throws JMSException {
 
        final ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(USER, PASSWORD, URL);
        final Connection connection = connectionFactory.createConnection();
 
        // Add ConsoleHandler to Logger.
		CustomRecordFormatter formatter = new CustomRecordFormatter();
		ConsoleHandler consoleHandler = new ConsoleHandler();
		consoleHandler.setFormatter(formatter);
		logger.setUseParentHandlers(false);
		logger.addHandler(consoleHandler);
		logger.info("Inicialitzant node Manager ");
		
        connection.start();
 
        final Session session = connection.createSession(TRANSACTED_SESSION, Session.AUTO_ACKNOWLEDGE);
        final Destination destination = session.createQueue(DESTINATION_QUEUE);
        final MessageConsumer consumer = session.createConsumer(destination);
 
        int index = 1;
        while (index > 0) {

        	processAllMessagesInQueue(consumer);
			
			logger.info("Hearbeat cicle node Manager");
			// Stays alive for ten seconds.
			try {
				Thread.sleep(10000L);
			} catch (InterruptedException e) {
				;
			}
			
		}
        
        consumer.close();
        session.close();
        connection.close();
 
        showProcessedResults();
    }
 
    /**
     * Process all messages in queue.
     *
     * @param consumer the consumer
     * @throws JMSException the jMS exception
     */
    private void processAllMessagesInQueue(MessageConsumer consumer) throws JMSException {
        Message message;
        while ((message = consumer.receive(TIMEOUT)) != null) {
            proccessMessage(message);
        }
    }
 
    /**
     * Proccess message.
     *
     * @param message the message
     * @throws JMSException the jMS exception
     */
    private void proccessMessage(Message message) throws JMSException {
        if (message instanceof TextMessage) {
            final TextMessage textMessage = (TextMessage) message;
            final String text = textMessage.getText();
            incrementMessageType(text);
            totalConsumedMessages++;
        }
        if (message instanceof ObjectMessage) {
            final ObjectMessage objectMessage = (ObjectMessage) message;
            final MessageSimpleMQ msgMQ = (MessageSimpleMQ) objectMessage.getObject();
            logger.info("Treball en execuci >> "+msgMQ.getIdjob()+"-"+msgMQ.getOffsetjob()+"> " + msgMQ.getCommand() +
            		"/" + msgMQ.getTimestamp());
            StartExec exec = new StartExec(msgMQ);
            totalConsumedMessages++;
        }
    }
 
    /**
     * Increment message type.
     *
     * @param message the message
     */
    private void incrementMessageType(String message) {
        if (consumedMessageTypes.get(message) == null) {
            consumedMessageTypes.put(message, 1);
        } else {
            final int numberOfTypeMessages = consumedMessageTypes.get(message);
            consumedMessageTypes.put(message, numberOfTypeMessages + 1);
        }
    }
 
    /**
     * Show processed results.
     */
    private void showProcessedResults() {
    	logger.info("Processats un total de " + totalConsumedMessages + " missatges");
    }
 
    /**
     * The main method.
     *
     * @param args the arguments
     * @throws JMSException the jMS exception
     */
    public static void main(String[] args) throws JMSException {
        final ClientManager userActionConsumer = new ClientManager();
        userActionConsumer.processMessages();
    }
}