package uoc.ows.nodeclient;

import org.apache.activemq.ActiveMQConnection;
import org.apache.activemq.ActiveMQConnectionFactory;
 
import javax.jms.*;

import uoc.ows.beans.MessageExecutionMQ;

// TODO: Auto-generated Javadoc
/**
 * MessageExecutionSender
 * s tracta de la classe que retorna els missatges propis de la execuci (stdout/err) i el registre del seu resultat.
 *
 * @author LLuis Cozar Prez
 */
public class MessageExecutionSender {
 
    /** The Constant USER. */
    private static final String USER = ActiveMQConnection.DEFAULT_USER;
 
    /** The Constant PASSWORD. */
    private static final String PASSWORD = ActiveMQConnection.DEFAULT_PASSWORD;
 
    /** The Constant TRANSACTED_SESSION. */
    private static final boolean TRANSACTED_SESSION = true;
 
    /**
     * Send messages.
     *
     * @param msgMQ 
     * @param type 
     * @throws JMSException 
     */
    public void sendMessages(MessageExecutionMQ msgMQ, String type) throws JMSException {
    	
    	String URL;
    	String DESTINATION_QUEUE; 
    	
    	if (type == "SEND") {
    		URL = "tcp://" + msgMQ.getReceiverNode() + ":61616";
        	DESTINATION_QUEUE = msgMQ.getQname();  		
    	}
    	else { /** RECEIVE **/
    		URL = "tcp://" + msgMQ.getSenderNode() + ":61616";
    		DESTINATION_QUEUE = msgMQ.getQnameReply();
    	}
    	
        final ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(USER, PASSWORD, URL);
        Connection connection = connectionFactory.createConnection();
        connection.start();
 
        final Session session = connection.createSession(TRANSACTED_SESSION, Session.AUTO_ACKNOWLEDGE);
        final Destination destination = session.createQueue(DESTINATION_QUEUE);
 
        final MessageProducer producer = session.createProducer(destination);
        producer.setDeliveryMode(DeliveryMode.PERSISTENT);
        
        sendMessages(session, producer, msgMQ);
        
        session.commit();
 
        session.close();
        connection.close();
        
    }
 
    /**
     * Send messages.
     *
     * @param session 
     * @param producer 
     * @param msgMQ 
     * @throws JMSException 
     */
    private void sendMessages(Session session, MessageProducer producer, MessageExecutionMQ msgMQ) throws JMSException {
        final MessageExecutionSender messageSender = new MessageExecutionSender();
        messageSender.sendMessage(msgMQ, session, producer);
    }
 
    
    /**
     * Send message.
     *
     * @param msgMQ 
     * @param session 
     * @param producer 
     * @throws JMSException
     */
    private void sendMessage(MessageExecutionMQ msgMQ, Session session, MessageProducer producer) throws JMSException {
        final ObjectMessage objectMessage = session.createObjectMessage(msgMQ);
        producer.send(objectMessage);
    }
 
}