package uoc.ows.nodeclient;

import java.util.logging.Logger;

import javax.jms.JMSException;

import uoc.ows.common.*;
import uoc.ows.server.MessageSender;
import uoc.ows.server.ServerManager;
import uoc.ows.beans.MessageSimpleMQ;
import uoc.ows.beans.MessageExecutionMQ;

/**
 * StartExec
 * s tracta de la classe que gestiona la propia execuci del comandament sollicitat pel scheduler
 * Cada registre corresponent al stdout/err viatge cap al scheduler mitjanant altre missatge aix com el registre
 * identificatiu de finalitzaci correcte o incorrecte del treball.
 *
 * @author LLuis Cozar Prez
 */
public class StartExec
{
	
	/** The logger. */
	private static Logger logger = Logger.getLogger(ServerManager.class.getName());
	
    /**
     * Instantiates a new start exec.
     *
     * @param msgMQ 
     */
    public StartExec(MessageSimpleMQ msgMQ)
    {    

    	MessageExecutionMQ msgExecutionMQ = new MessageExecutionMQ();
    	msgExecutionMQ.setIdjob(msgMQ.getIdjob());
    	msgExecutionMQ.setOffsetjob(msgMQ.getOffsetjob());
    	msgExecutionMQ.setQname(msgMQ.getQname());
    	msgExecutionMQ.setQnameReply(msgMQ.getQnameReply());
    	msgExecutionMQ.setReceiverNode(msgMQ.getReceiverNode());
    	msgExecutionMQ.setSenderNode(msgMQ.getSenderNode());
    	msgExecutionMQ.setTimestamp(msgMQ.getTimestamp());
    	
        try
        {
            String cmd = msgMQ.getCommand();
            String environment = msgMQ.getEnvironment();
            String[] strArray = new String[] {environment};
            String parms = msgMQ.getParameters();
            String cmdComplex;
            if ((parms != null) && (parms != "") ) {
            	cmdComplex = cmd + " " + parms; 
            }
            else cmdComplex = cmd;  
            
            Runtime rt = Runtime.getRuntime();
            Process proc = rt.exec(cmdComplex, strArray);
            
            // any error message?
            StreamGobbler errorGobbler = new 
                StreamGobbler(proc.getErrorStream(), "ERR", msgExecutionMQ);            
            
            // any output?
            StreamGobbler outputGobbler = new 
            StreamGobbler(proc.getInputStream(), "OUT", msgExecutionMQ);
                
            // kick them off
            errorGobbler.start();
            outputGobbler.start();
                                    
            // any error???
            int exitVal = proc.waitFor();
            
            msgMQ.setReturnCode(exitVal);
            sendReply(msgMQ);
            
            logger.info("Treball finalitzat >> "+msgMQ.getIdjob()+"-"+msgMQ.getOffsetjob()+"-RC=" + exitVal +
            		"/" + msgMQ.getTimestamp());
            
        } catch (Throwable t)
          {
        	
        	msgExecutionMQ.setPosition(1);
            msgExecutionMQ.setLine(t.getMessage());
            sendReply(msgExecutionMQ);
            
            int exitVal = 16;
            
            msgMQ.setReturnCode(exitVal);
            sendReply(msgMQ);
            
            logger.info("Treball finalitzat amb ERRORS! >> "+msgMQ.getIdjob()+"-"+msgMQ.getOffsetjob()+"-RC=" + exitVal +
            		"/" + msgMQ.getTimestamp() + ">> "+t.getMessage());
            
          }
    }
    
    /**
     * Send reply.
     *
     * @param msgMQ 
     */
    private void sendReply(MessageSimpleMQ msgMQ){
    	try {	
	    	final MessageSender messageSender = new MessageSender();
		    messageSender.sendMessages(msgMQ, "RECEIVE");
	    } catch (JMSException e) { 
	    	
	    }
    }
    
    /**
     * Send reply.
     *
     * @param msgMQ 
     */
    private void sendReply(MessageExecutionMQ msgMQ){
    	try {	
    		final MessageExecutionSender messageSender = new MessageExecutionSender();
    		messageSender.sendMessages(msgMQ, "RECEIVE");
	    } catch (JMSException e) { 
	    	
	    }
    }
    
}
