package uoc.ows.server;

import org.apache.activemq.ActiveMQConnection;
import org.apache.activemq.ActiveMQConnectionFactory;
 
import javax.jms.*;
import java.util.logging.Logger;

import uoc.ows.bbdd.GestorJobs;
import uoc.ows.bbdd.GestorExecutions;
import uoc.ows.bbdd.GestorTransitions;
import uoc.ows.beans.Execution;
import uoc.ows.beans.Job;
import uoc.ows.beans.MessageSimpleMQ;
import uoc.ows.beans.MessageExecutionMQ;
import uoc.ows.common.Constants;
import uoc.ows.common.OWSException;

/**
 * MessagesReceiver
 * Classe dedicada a la recuperaci dels missatges de la resta de nodes de la xara
 * Recupera tant missatges relatius a la finalitzacin com stdout/err de les execucions
 * 
 * @author Lluis Cozar Prez
 *
 */
public class MessageReceiver {
 
    private static final String URL = "tcp://localhost:61616";
 
    private static final String USER = ActiveMQConnection.DEFAULT_USER;
 
    private static final String PASSWORD = ActiveMQConnection.DEFAULT_PASSWORD;
 
    private static final String DESTINATION_QUEUE = Constants.qnameReply;
 
    private static final boolean TRANSACTED_SESSION = false;
 
    private static final int TIMEOUT = 1000;
 
    private int totalConsumedMessages = 0;
    
    private Logger logger = Logger.getLogger(ServerManager.class.getName());
    
    private GestorJobs gu;
    private GestorTransitions gt;
    private GestorExecutions ge;
    
    /**
     * MessagesReceiver 
     * Constructor MessageRecevier amb connexio parametritzada
     * 
     * @param gu
     * @throws OWSException
     */
    public MessageReceiver(GestorJobs gu) throws OWSException {
        this.gu = gu;
        gt = new GestorTransitions(gu.getConexion());
        ge = new GestorExecutions(gu.getConexion());
    }
 
    /**
     * processMessages
     * Recupera la totalitat de missatges existents a les cues i els hi dona tractament
     * @throws JMSException
     * @throws OWSException
     */
    public void processMessages() throws JMSException, OWSException {
 
        final ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(USER, PASSWORD, URL);
        final Connection connection = connectionFactory.createConnection();
 
        connection.start();
 
        final Session session = connection.createSession(TRANSACTED_SESSION, Session.AUTO_ACKNOWLEDGE);
        final Destination destination = session.createQueue(DESTINATION_QUEUE);
        final MessageConsumer consumer = session.createConsumer(destination);
 
        processAllMessagesInQueue(consumer);
 
        consumer.close();
        session.close();
        connection.close();
 
        showProcessedResults();
    }
 
    private void processAllMessagesInQueue(MessageConsumer consumer) throws JMSException, OWSException {
        Message message;
        while ((message = consumer.receive(TIMEOUT)) != null) {
            proccessMessage(message);
        }
    }
 
    private void proccessMessage(Message message) throws JMSException, OWSException {
        if (message instanceof ObjectMessage) {
            final ObjectMessage objectMessage = (ObjectMessage) message;
            
            if (objectMessage.getObject() instanceof MessageSimpleMQ) {
            	final MessageSimpleMQ msgMQ = (MessageSimpleMQ) objectMessage.getObject();
            
            	Job wrkJob = gu.getJob(msgMQ.getIdjob());
            	wrkJob.setLastRC(msgMQ.getReturnCode());
            	
            	int nexec = wrkJob.getNumberofExecutions();
            	nexec++;
            	wrkJob.setNumberofExecutions(nexec);
            	
            	if (wrkJob.getLastRC() == 0) {
            	   wrkJob.setIdstatus(Constants.FOK);
            	}
            	else {
            	   wrkJob.setIdstatus(Constants.KO);
            	}
            	gu.updateJob(wrkJob);
            	gt.insertTransition(wrkJob);
            	totalConsumedMessages++;
            }
            if (objectMessage.getObject() instanceof MessageExecutionMQ) {
            	final MessageExecutionMQ msgMQ = (MessageExecutionMQ) objectMessage.getObject();
                Execution executionLine = new Execution();
                executionLine.setIdjob(msgMQ.getIdjob());
                executionLine.setOffsetjob(msgMQ.getOffsetjob());
                executionLine.setLine(msgMQ.getLine());
                executionLine.setPosition(msgMQ.getPosition());
                executionLine.setTimestamp(msgMQ.getTimestamp()); /** now*/
            	ge.insertLineExecution(executionLine);
            	totalConsumedMessages++;
            }
        }
    }
 
    private void showProcessedResults() {
    	logger.info("Processats un total de " + totalConsumedMessages + " missatges");
    }
 
}