package uoc.ows.server;

import org.apache.activemq.ActiveMQConnection;
import org.apache.activemq.ActiveMQConnectionFactory;
 
import javax.jms.*;

import uoc.ows.beans.MessageSimpleMQ;
 
/**
 * MessageSender.
 * Es tracta de la classe encarregada de l'enviament de missatges mitjanant la xarxa Active MQ
 *
 * @author Lluis Cozar Prez
 *
 */
public class MessageSender {
 
    private static final String USER = ActiveMQConnection.DEFAULT_USER;
 
    private static final String PASSWORD = ActiveMQConnection.DEFAULT_PASSWORD;
 
    private static final boolean TRANSACTED_SESSION = true;
 
    /**
     * Send messages.
     * Enviament de missatges de solicitud d'execuci mitjanant el bean MessageSimpleMQ 
     *
     * @param msgMQ
     * @param type
     * @throws JMSException
     */
    public void sendMessages(MessageSimpleMQ msgMQ, String type) throws JMSException {
    	
    	String URL;
    	String DESTINATION_QUEUE; 
    	
    	if (type == "SEND") {
    		URL = "tcp://" + msgMQ.getReceiverNode() + ":61616";
        	DESTINATION_QUEUE = msgMQ.getQname();  		
    	}
    	else { /** RECEIVE **/
    		URL = "tcp://" + msgMQ.getSenderNode() + ":61616";
    		DESTINATION_QUEUE = msgMQ.getQnameReply();
    	}
    	
        final ActiveMQConnectionFactory connectionFactory = new ActiveMQConnectionFactory(USER, PASSWORD, URL);
        Connection connection = connectionFactory.createConnection();
        connection.start();
 
        final Session session = connection.createSession(TRANSACTED_SESSION, Session.AUTO_ACKNOWLEDGE);
        final Destination destination = session.createQueue(DESTINATION_QUEUE);
 
        final MessageProducer producer = session.createProducer(destination);
        producer.setDeliveryMode(DeliveryMode.PERSISTENT);
        
        sendMessages(session, producer, msgMQ);
        
        session.commit();
 
        session.close();
        connection.close();
        
    }
 
    private void sendMessages(Session session, MessageProducer producer, MessageSimpleMQ msgMQ) throws JMSException {
        final MessageSender messageSender = new MessageSender();
        messageSender.sendMessage(msgMQ, session, producer);
    }
 
    
    private void sendMessage(MessageSimpleMQ msgMQ, Session session, MessageProducer producer) throws JMSException {
        final ObjectMessage objectMessage = session.createObjectMessage(msgMQ);
        producer.send(objectMessage);
    }
 
}