package uoc.ows.server;

import java.util.logging.Logger;

import uoc.ows.bbdd.GestorJobs;
import uoc.ows.bbdd.GestorTransitions;
import uoc.ows.common.OWSException;
import uoc.ows.server.ActiveMqTask;
import uoc.ows.common.Constants;
import uoc.ows.beans.Job;

import it.sauronsoftware.cron4j.SchedulerListener;
import it.sauronsoftware.cron4j.TaskExecutor;

/**
 * MySchedulerListener
 * Listener per interceptar els canvis d'estats dels jobs planificats 
 * 
 * @author LLuis Cozar Prez
 */
public class MySchedulerListener implements SchedulerListener {

	private Logger logger = Logger.getLogger(ServerManager.class.getName());
    private GestorJobs gu;
    private GestorTransitions gt;
    
	/**
	 * Instantiates a new my scheduler listener.
	 *
	 * @param gu
	 * @throws OWSException
	 */
	public MySchedulerListener(GestorJobs gu) throws OWSException {
		// Obtains Logger instance
		logger.info("Inicialitzant SchedulerListener");
		this.gu = gu;
		this.gt = new GestorTransitions(gu.getConexion());
		
	}
	
	
	/**
	 * Interceptem quan un treball es llana i provoquem el canvi d'estat.
	 *
	 * (non-Javadoc)
	 * @see it.sauronsoftware.cron4j.SchedulerListener#taskLaunching(it.sauronsoftware.cron4j.TaskExecutor)
	 */
	public void taskLaunching(TaskExecutor executor) {

		ActiveMqTask task = (ActiveMqTask) executor.getTask();
		logger.info("Job " + task.getJob().getIdjob() + " inici submit");
		task.job.setIdstatus(Constants.SUBMIT_REQUEST);
		
		try {
			Job wrkJob = gu.getJob(task.getJob().getIdjob());
			wrkJob.setIdstatus(Constants.SUBMIT_REQUEST);
			gu.updateJob(wrkJob);
			gt.insertTransition(wrkJob);
		} catch (OWSException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	
	}

	/**
	 * Interceptem quan un treball finalitza OK i provquem el canvi d'estat.
	 * 
	* (non-Javadoc)
	 * @see it.sauronsoftware.cron4j.SchedulerListener#taskLaunching(it.sauronsoftware.cron4j.TaskExecutor)
	 */
	public void taskSucceeded(TaskExecutor executor) {
		ActiveMqTask task = (ActiveMqTask) executor.getTask();
		logger.info("Job " + task.getJob().getIdjob() + " demanat submit a plataforma "
		+ task.getJob().getIdnode() + " OK");
		
		task.job.setIdstatus(Constants.SUBMIT_IN_PROGRESS);
		
		try {
			Job wrkJob = gu.getJob(task.getJob().getIdjob());
			wrkJob.setIdstatus(Constants.SUBMIT_IN_PROGRESS);
			gu.updateJob(wrkJob);
			gt.insertTransition(wrkJob);
		} catch (OWSException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Interceptem quan un treball finalitza incorrectament i provquem el canvi d'estat.
	 * 
	* (non-Javadoc)
	 * @see it.sauronsoftware.cron4j.SchedulerListener#taskLaunching(it.sauronsoftware.cron4j.TaskExecutor)
	 */
	public void taskFailed(TaskExecutor executor, Throwable exception) {
		ActiveMqTask task = (ActiveMqTask) executor.getTask();
		logger.info("Job " + task.getJob().getIdjob() + " submit failed!");
		exception.printStackTrace();
		
		task.job.setIdstatus(Constants.SUBMIT_FAILED);
		
		try {
			Job wrkJob = gu.getJob(task.getJob().getIdjob());
			wrkJob.setIdstatus(Constants.SUBMIT_FAILED);
			gu.updateJob(wrkJob);
		} catch (OWSException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
